package tk.eclipse.plugin.jseditor.editors;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DefaultIndentLineAutoEditStrategy;
import org.eclipse.jface.text.DocumentCommand;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;

import tk.eclipse.plugin.htmleditor.HTMLPlugin;

/**
 * Provides auto inserting for JavaScript.
 * 
 * @author Naoki Takezoe
 * @since 2.0.3
 */
public class JavaScriptAutoEditStrategy extends DefaultIndentLineAutoEditStrategy {
	
	protected boolean enable;
	
	public JavaScriptAutoEditStrategy(){
		IPreferenceStore store = HTMLPlugin.getDefault().getPreferenceStore();
		this.enable = store.getBoolean(HTMLPlugin.PREF_AUTO_EDIT);
	}
	
	/**
	 * @since 2.0.5
	 */
	public void setEnabled(boolean enable){
		this.enable = enable;
	}
	
	public void customizeDocumentCommand(IDocument document, DocumentCommand command) {
		if(enable){
			String text = command.text;
			if(text.equals("{")){
				append(command, "}", 1);
				return;
			}
			if(text.equals("(")){
				append(command, ")", 1);
				return;
			}
			if(text.equals("\"")){
				append(command, "\"", 1);
				return;
			}
			if(text.equals("'")){
				append(command, "'", 1);
				return;
			}
			if(text.equals("\r\n") || text.equals("\n") || text.equals("\r")){
				try {
					String line = getLine(document, command.offset);
					String trim = line.trim();
					if(trim.endsWith("/*") || trim.endsWith("/**")){
						String indent = getIndent(line);
						append(command, indent + " * " + text + indent + " */", indent.length() + 5);
					}
					if(trim.startsWith("*")){
						String indent = getIndent(line);
						append(command, indent + "* ", indent.length() + 4);
					}
				} catch(BadLocationException ex){
				}
			}
		}
		super.customizeDocumentCommand(document, command);
	}
	
	private static String getLine(IDocument document, int offset) throws BadLocationException {
		int line = document.getLineOfOffset(offset);
		IRegion reg = document.getLineInformation(line);
		return document.get(reg.getOffset(), reg.getLength());
	}

	
	private static String getIndent(String line) throws BadLocationException {
		StringBuilder sb = new StringBuilder();
		for(int i=0;i<line.length();i++){
			char c = line.charAt(i);
			if(c==' ' || c=='\t'){
				sb.append(c);
			} else {
				break;
			}
		}
		return sb.toString();
	}
	
	private void append(DocumentCommand command, String append, int position){
		command.text = command.text + append;
		command.doit = false;
		command.shiftsCaret = false;
		command.caretOffset = command.offset + position;
	}
	
	
}
